/* C Interface example for ASL_zfc2fb , ASL_zfc2bf */
#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include "mkl_dfti.h"
int main()
{
  int nx = 5; int ny = 4; int nz = 3;
  int lx = 5; int ly = 4; int lz = 3;
  MKL_Complex8 *c;
  int i,j,k;
  DFTI_DESCRIPTOR *my_desc1_handle;
  long status;
  long dm[3]={nz, ny, nx};   /* MKL user may write {nx, ny, nz}. */

  printf( " *** ASL_zfc3fb , ASL_zfc3bf ***\n" );
  printf( "\n ** Input **\n" );
  c = (MKL_Complex8 *)malloc((size_t)( sizeof(MKL_Complex8) * (lx*ly*lz) ));

  if( c == NULL )
  {
    printf( "no enough memory for array c\n" );
    return -1;
  }

  printf( "\tnx = %6d\n", nx );
  printf( "\tny = %6d\n", ny );
  printf( "\tny = %6d\n", nz );

  for( k=1 ; k<=nz ; k++ )
  {
    for( j=1 ; j<=ny ; j++ )
    {
      for( i=1 ; i<=nx ; i++ )
      {
        c[(i-1)+lx*(j-1)+lx*ly*(k-1)].real=(float)(nx+ny+nz)/(float)(i+j+k) ;
        c[(i-1)+lx*(j-1)+lx*ly*(k-1)].imag=(float)(i*j*k)/(float)(nx*ny*nz) ;
      }
    }
  }

  printf( "\tc[ix][iy][1]\n" );
  for( i=0 ; i<nx ; i++ )
  {
    for( j=0 ; j<ny ; j++ )
    {
      printf( "\t(%8.3g,%8.3g)", c[i+lx*j ].real, c[i+lx*j ].imag );
    }
    printf( "\n" );
  }
  printf( "\tc[ix][iy][2]\n" );
  for( i=0 ; i<nx ; i++ )
  {
    for( j=0 ; j<ny ; j++ )
    {
      printf( "\t(%8.3g,%8.3g)", c[i+lx*j+lx*ly*1].real, c[i+lx*j+lx*ly*1].imag );
    }
    printf( "\n" );
  }
  printf( "\tc[ix][iy][3]\n" );
  for( i=0 ; i<nx ; i++ )
  {
    for( j=0 ; j<ny ; j++ )
    {
      printf( "\t(%8.3g,%8.3g)", c[i+lx*j+lx*ly*2].real, c[i+lx*j+lx*ly*2].imag );
    }
  printf( "\n" );
  }

  status=DftiCreateDescriptor( &my_desc1_handle, DFTI_SINGLE, DFTI_COMPLEX, 3, dm);
  status=DftiCommitDescriptor( my_desc1_handle );
  status=DftiComputeForward( my_desc1_handle, c);
  status=DftiFreeDescriptor( &my_desc1_handle );

  for( i=0 ; i<lx*ly*lz ; i++)
  {
    c[i].real /= (float)( nx * ny * nz );
    c[i].imag /= (float)( nx * ny * nz );
  }

  printf( "\n ** Output **\n" );
  printf( "\t< Forward Transform >\n" );

  printf( "\tc[ix][iy][1]\n" );
  for( i=0 ; i<nx ; i++ )
  {
    for( j=0 ; j<ny ; j++ )
    {
      printf( "\t(%8.3g,%8.3g)", c[i+lx*j ].real, c[i+lx*j ].imag );
    }
    printf( "\n" );
  }
  printf( "\tc[ix][iy][2]\n" );
  for( i=0 ; i<nx ; i++ )
  {
    for( j=0 ; j<ny ; j++ )
    {
      printf( "\t(%8.3g,%8.3g)", c[i+lx*j+lx*ly*1].real, c[i+lx*j+lx*ly*1].imag );
    }
    printf( "\n" );
  }
  printf( "\tc[ix][iy][3]\n" );
  for( i=0 ; i<nx ; i++ )
  {
    for( j=0 ; j<ny ; j++ )
    {
      printf( "\t(%8.3g,%8.3g)", c[i+lx*j+lx*ly*2].real, c[i+lx*j+lx*ly*2].imag );
    }
    printf( "\n" );
  }

  status=DftiCreateDescriptor( &my_desc1_handle, DFTI_SINGLE, DFTI_COMPLEX, 3, dm);
  status=DftiCommitDescriptor( my_desc1_handle );
  status=DftiComputeBackward( my_desc1_handle, c);
  status=DftiFreeDescriptor( &my_desc1_handle );

  printf( "\t< Backward Transform >\n" );

  printf( "\tc[ix][iy][1]\n" );
  for( i=0 ; i<nx ; i++ )
  {
    for( j=0 ; j<ny ; j++ )
    {
      printf( "\t(%8.3g,%8.3g)", c[i+lx*j ].real, c[i+lx*j ].imag );
    }
    printf( "\n" );
  }
  printf( "\tc[ix][iy][2]\n" );
  for( i=0 ; i<nx ; i++ )
  {
    for( j=0 ; j<ny ; j++ )
    {
      printf( "\t(%8.3g,%8.3g)", c[i+lx*j+lx*ly*1].real, c[i+lx*j+lx*ly*1].imag );
    }
    printf( "\n" );
  }
  printf( "\tc[ix][iy][3]\n" );
  for( i=0 ; i<nx ; i++ )
  {
    for( j=0 ; j<ny ; j++ )
    {
      printf( "\t(%8.3g,%8.3g)", c[i+lx*j+lx*ly*2].real, c[i+lx*j+lx*ly*2].imag );
    }
    printf( "\n" );
  }

  free(c);
  return 0;
}
