/**
 * \license This project is released under the BSD 2-clause license
 *
 * Copyright (C) 2014 Xevolver Project. All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * - Redistributions of source code must retain the above copyright
 *   notice, this list of conditions and the following disclaimer.
 *
 * - Redistributions in binary form must reproduce the above copyright
 *   notice, this list of conditions and the following disclaimer in
 *   the documentation and/or other materials provided with the
 *   distribution.
 *    
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 */
/* Ѵåѷ */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#ifndef MKL_ASLI32_INCLUDED
#include "mkl_asli32.h"
#endif

#ifndef UTIL_INCLUDED
#include "util.h"
#endif

  /* Fortran */
Int32 dfti_compute_backward_c_(struct MKL2ASL_DESCRIPTOR_S **desc, Complex8 *inoutdt)
{
  struct MKL2ASL_DESCRIPTOR_S *desc_p = *desc;
  Int32  asl_ifax[20*desc_p->dim];
  Int32  asl_err;
  Int32  asl_isw = (-1);  /* 1:Ѵ-1:Ѵ */
  Int32  *len = desc_p->length;

  if (desc_p->commit_flag != MKL2ASL_COMMITTED)
  {
    fprintf(stderr, "Descriptor is not committed\n");
    return MKL2ASL_INCONSISTENT_CONFIGURATION;
  }

  if (desc_p->placement_flag != MKL2ASL_INPLACE)
  {
    fprintf(stderr, "PLACEMENT must be INPLACE\n");
    return MKL2ASL_INCONSISTENT_CONFIGURATION;
  }

  if ( desc_p->dim < 1 )
  {
    fprintf(stderr,"Invalid dimension.\n");
    return MKL2ASL_BAD_DESCRIPTOR;
  }
  else if ( desc_p->dim == 1 ) {
    Complex8 asl_wk[len[0]];
    float    asl_trigs[2*len[0]];

    cfc1fb_(&len[0], inoutdt, &len[0], &asl_isw, asl_ifax, asl_trigs, asl_wk, &asl_err);
  }
  else if ( desc_p->dim == 2 ) {
    Complex8 asl_wk[len[0]*len[1]];
    float    asl_trigs[2*(len[0]+len[1])];

    cfc2fb_(&len[0], &len[1], inoutdt, &len[0], &len[1], &asl_isw, asl_ifax, asl_trigs, asl_wk, &asl_err);
  }
  else if ( desc_p->dim == 3 ) {
    Complex8 asl_wk[len[0]*len[1]*len[2]];
    float    asl_trigs[2*(len[0]+len[1]+len[2])];

    cfc3fb_(&len[0], &len[1], &len[2], inoutdt, &len[0], &len[1], &len[2], &asl_isw, asl_ifax, asl_trigs, asl_wk, &asl_err);
  }
  else
  {
    fprintf(stderr,"More than 3 dimensions are not supported.\n");
    return MKL2ASL_BAD_DESCRIPTOR;
  }

  return __MKL2ASL_check_asl_error(asl_err);
}

#if 0
  /* Fortran */
Int32 dfti_compute_backward_cc_(struct MKL2ASL_DESCRIPTOR_S **desc, Complex8 *indt, Complex8 *outdt)
{
  struct MKL2ASL_DESCRIPTOR_S *desc_p = *desc;
  Int32  asl_ifax[20*desc_p->dim];
  Int32  asl_err;
  Int32  asl_isw = (-1);  /* 1:Ѵ-1:Ѵ */
  Int32  *len = desc_p->length;

  if (desc_p->commit_flag != MKL2ASL_COMMITTED)
  {
    fprintf(stderr, "Descriptor is not committed\n");
    return MKL2ASL_INCONSISTENT_CONFIGURATION;
  }

  if (desc_p->placement_flag != MKL2ASL_NOT_INPLACE)
  {
    fprintf(stderr, "PLACEMENT must be NOT_INPLACE\n");
    return MKL2ASL_INCONSISTENT_CONFIGURATION;
  }

  if ( desc_p->dim < 1 )
  {
    fprintf(stderr,"Invalid dimension.\n");
    return MKL2ASL_BAD_DESCRIPTOR;
  }
  else if ( desc_p->dim == 1 ) {
    Complex8 asl_wk[len[0]];
    float    asl_trigs[2*len[0]];

    memmove(outdt,indt,len[0]*sizeof(Complex8));
    cfc1fb_(&len[0], outdt, &len[0], &asl_isw, asl_ifax, asl_trigs, asl_wk, &asl_err);
  }
  else if ( desc_p->dim == 2 ) {
    Complex8 asl_wk[len[0]*len[1]];
    float    asl_trigs[2*(len[0]+len[1])];

    memmove(outdt,indt,len[0]*len[1]*sizeof(Complex8));
    cfc2fb_(&len[0], &len[1], outdt, &len[0], &len[1], &asl_isw, asl_ifax, asl_trigs, asl_wk, &asl_err);
  }
  else if ( desc_p->dim == 3 ) {
    Complex8 asl_wk[len[0]*len[1]*len[2]];
    float    asl_trigs[2*(len[0]+len[1]+len[2])];

    memmove(outdt,indt,len[0]*len[1]*len[2]*sizeof(Complex8));
    cfc3fb_(&len[0], &len[1], &len[2], outdt, &len[0], &len[1], &len[2], &asl_isw, asl_ifax, asl_trigs, asl_wk, &asl_err);
  }
  else
  {
    fprintf(stderr,"More than 3 dimensions are not supported.\n");
    return MKL2ASL_BAD_DESCRIPTOR;
  }

  return __MKL2ASL_check_asl_error(asl_err);
}
#endif

  /* Fortran */
Int32 dfti_compute_backward_z_(struct MKL2ASL_DESCRIPTOR_S **desc, Complex16 *inoutdt)
{
  struct MKL2ASL_DESCRIPTOR_S *desc_p = *desc;
  Int32  asl_ifax[20*desc_p->dim];
  Int32  asl_err;
  Int32  asl_isw = (-1);  /* 1:Ѵ-1:Ѵ */
  Int32  *len = desc_p->length;

  if (desc_p->commit_flag != MKL2ASL_COMMITTED)
  {
    fprintf(stderr, "Descriptor is not committed\n");
    return MKL2ASL_INCONSISTENT_CONFIGURATION;
  }

  if (desc_p->placement_flag != MKL2ASL_INPLACE)
  {
    fprintf(stderr, "PLACEMENT must be INPLACE\n");
    return MKL2ASL_INCONSISTENT_CONFIGURATION;
  }

  if ( desc_p->dim < 1 )
  {
    fprintf(stderr,"Invalid dimension.\n");
    return MKL2ASL_BAD_DESCRIPTOR;
  }
  else if ( desc_p->dim == 1 ) {
    Complex16 asl_wk[len[0]];
    double    asl_trigs[2*len[0]];

    zfc1fb_(&len[0], inoutdt, &len[0], &asl_isw, asl_ifax, asl_trigs, asl_wk, &asl_err);
  }
  else if ( desc_p->dim == 2 ) {
    Complex16 asl_wk[len[0]*len[1]];
    double    asl_trigs[2*(len[0]+len[1])];

    zfc2fb_(&len[0], &len[1], inoutdt, &len[0], &len[1], &asl_isw, asl_ifax, asl_trigs, asl_wk, &asl_err);
  }
  else if ( desc_p->dim == 3 ) {
    Complex16 asl_wk[len[0]*len[1]*len[2]];
    double    asl_trigs[2*(len[0]+len[1]+len[2])];

    zfc3fb_(&len[0], &len[1], &len[2], inoutdt, &len[0], &len[1], &len[2], &asl_isw, asl_ifax, asl_trigs, asl_wk, &asl_err);
  }
  else
  {
    fprintf(stderr,"More than 3 dimensions are not supported.\n");
    return MKL2ASL_BAD_DESCRIPTOR;
  }

  return __MKL2ASL_check_asl_error(asl_err);
}

#if 0
  /* Fortran */
Int32 dfti_compute_backward_zz_(struct MKL2ASL_DESCRIPTOR_S **desc, Complex16 *indt, Complex16 *outdt)
{
  struct MKL2ASL_DESCRIPTOR_S *desc_p = *desc;
  Int32  asl_ifax[20*desc_p->dim];
  Int32  asl_err;
  Int32  asl_isw = (-1);  /* 1:Ѵ-1:Ѵ */
  Int32  *len = desc_p->length;

  if (desc_p->commit_flag != MKL2ASL_COMMITTED)
  {
    fprintf(stderr, "Descriptor is not committed\n");
    return MKL2ASL_INCONSISTENT_CONFIGURATION;
  }

  if (desc_p->placement_flag != MKL2ASL_NOT_INPLACE)
  {
    fprintf(stderr, "PLACEMENT must be NOT_INPLACE\n");
    return MKL2ASL_INCONSISTENT_CONFIGURATION;
  }

  if ( desc_p->dim < 1 )
  {
    fprintf(stderr,"Invalid dimension.\n");
    return MKL2ASL_BAD_DESCRIPTOR;
  }
  else if ( desc_p->dim == 1 ) {
    Complex16 asl_wk[len[0]];
    double    asl_trigs[2*len[0]];

    memmove(outdt,indt,len[0]*sizeof(Complex16));
    zfc1fb_(&len[0], outdt, &len[0], &asl_isw, asl_ifax, asl_trigs, asl_wk, &asl_err);
  }
  else if ( desc_p->dim == 2 ) {
    Complex16 asl_wk[len[0]*len[1]];
    double    asl_trigs[2*(len[0]+len[1])];

    memmove(outdt,indt,len[0]*len[1]*sizeof(Complex16));
    zfc2fb_(&len[0], &len[1], outdt, &len[0], &len[1], &asl_isw, asl_ifax, asl_trigs, asl_wk, &asl_err);
  }
  else if ( desc_p->dim == 3 ) {
    Complex16 asl_wk[len[0]*len[1]*len[2]];
    double    asl_trigs[2*(len[0]+len[1]+len[2])];

    memmove(outdt,indt,len[0]*len[1]*len[2]*sizeof(Complex16));
    zfc3fb_(&len[0], &len[1], &len[2], outdt, &len[0], &len[1], &len[2], &asl_isw, asl_ifax, asl_trigs, asl_wk, &asl_err);
  }
  else
  {
    fprintf(stderr,"More than 3 dimensions are not supported.\n");
    return MKL2ASL_BAD_DESCRIPTOR;
  }

  return __MKL2ASL_check_asl_error(asl_err);
}
#endif
