/**
 * \license This project is released under the BSD 2-clause license
 *
 * Copyright (C) 2014 Xevolver Project. All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * - Redistributions of source code must retain the above copyright
 *   notice, this list of conditions and the following disclaimer.
 *
 * - Redistributions in binary form must reproduce the above copyright
 *   notice, this list of conditions and the following disclaimer in
 *   the documentation and/or other materials provided with the
 *   distribution.
 *    
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 */
/* Ѵåѷ */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#ifndef MKL_ASLI64_INCLUDED
#include "mkl_asli64.h"
#endif

#ifndef UTIL_INCLUDED
#include "util.h"
#endif

  /* C */
Int64 DftiComputeBackward(struct MKL2ASL_DESCRIPTOR_D *desc, void *inoutdt)
{
  Int64  asl_ifax[20*desc->dim];
  Int64  asl_err;
  Int64  asl_isw = (-1);  /* 1:Ѵ-1:Ѵ */
  Int64  *len = desc->length;

  if (desc->commit_flag != MKL2ASL_COMMITTED)
  {
    fprintf(stderr, "Descriptor is not committed\n");
    return MKL2ASL_INVALID_CONFIGURATION;
  }

  if (desc->placement_flag != MKL2ASL_INPLACE)
  {
    fprintf(stderr, "PLACEMENT must be INPLACE\n");
    return MKL2ASL_INCONSISTENT_CONFIGURATION;
  }

  if ( desc->dim < 1 )
  {
    fprintf(stderr,"Invalid dimension.\n");
    return MKL2ASL_BAD_DESCRIPTOR;
  }
  else if ( desc->dim == 1 ) {
    if ( desc->precision == MKL2ASL_SINGLE ) {
      Complex8  asl_wk[len[0]];
      float     asl_trigs[2*len[0]];

      asl_err = ASL_Lcfc1fb(len[0], inoutdt, len[0], asl_isw, asl_ifax, asl_trigs, asl_wk);
    }
    else {
      Complex16 asl_wk[len[0]];
      double    asl_trigs[2*len[0]];

      asl_err = ASL_Lzfc1fb(len[0], inoutdt, len[0], asl_isw, asl_ifax, asl_trigs, asl_wk);
    }
  }
  else if ( desc->dim == 2 ) {
    if ( desc->precision == MKL2ASL_SINGLE ) {
      Complex8  asl_wk[len[0]*len[1]];
      float     asl_trigs[2*(len[0]+len[1])];

      asl_err = ASL_Lcfc2fb(len[1], len[0], inoutdt, len[1], len[0], asl_isw, asl_ifax, asl_trigs, asl_wk);
    }
    else {
      Complex16 asl_wk[len[0]*len[1]];
      double    asl_trigs[2*(len[0]+len[1])];

      asl_err = ASL_Lzfc2fb(len[1], len[0], inoutdt, len[1], len[0], asl_isw, asl_ifax, asl_trigs, asl_wk);
    }
  }
  else if ( desc->dim == 3 ) {
    if ( desc->precision == MKL2ASL_SINGLE ) {
      Complex8  asl_wk[len[0]*len[1]*len[2]];
      float     asl_trigs[2*(len[0]+len[1]+len[2])];

      asl_err = ASL_Lcfc3fb(len[2], len[1], len[0], inoutdt, len[2], len[1], len[0], asl_isw, asl_ifax, asl_trigs, asl_wk);
    }
    else {
      Complex16 asl_wk[len[0]*len[1]*len[2]];
      double    asl_trigs[2*(len[0]+len[1]+len[2])];

      asl_err = ASL_Lzfc3fb(len[2], len[1], len[0], inoutdt, len[2], len[1], len[0], asl_isw, asl_ifax, asl_trigs, asl_wk);
    }
  }
  else
  {
    fprintf(stderr,"More than 3 dimensions are not supported.\n");
    return MKL2ASL_BAD_DESCRIPTOR;
  }

  return (Int64)__MKL2ASL_check_asl_error((Int32)asl_err);
}
